import os, sys
import argparse
import logging
from astropy.io import fits
import datetime
import shutil

from csst_dfs_api_local.common.db import DBClient
from csst_dfs_commons.utils.fits import get_header_value
from csst_dfs_commons.models.ifs import Level0Record

log = logging.getLogger('csst-dfs-api-local')

def ingest():
    
    parser = argparse.ArgumentParser(prog=f"{sys.argv[0]}", description="ingest the local files")
    parser.add_argument('-i','--infile', dest="infile", help="a file or a directory")
    parser.add_argument('-m', '--copyfiles', dest="copyfiles", action='store_true', default=False, help="whether copy files after import")
    args = parser.parse_args(sys.argv[1:])

    import_root_dir = args.infile
    if import_root_dir is None or (not os.path.isfile(import_root_dir) and not os.path.isdir(import_root_dir)):
        parser.print_help()
        sys.exit(0)

    db = DBClient()
    if os.path.isfile(import_root_dir):
        log.info(f"prepare import {import_root_dir}")
        ingest_one(import_root_dir, db, args.copyfiles)
    if os.path.isdir(import_root_dir):
        for (path, _, file_names) in os.walk(import_root_dir):
            for filename in file_names:
                if filename.find(".fits") > 0:
                    file_full_path = os.path.join(path, filename)
                    log.info(f"prepare import {file_full_path}")
                    try:
                        ingest_one(file_full_path, db, args.copyfiles)
                    except Exception as e:
                        print(f"{file_full_path} import error!!!")
                        log.error(e)

    db.close()

def ingest_one(file_path, db, copyfiles):
    dest_root_dir = os.getenv("CSST_LOCAL_FILE_ROOT", "/opt/temp/csst")

    hdul = fits.open(file_path)
    header = hdul[0].header
    #处理fits头
    obs_id = get_header_value("OBSID", header, "")
    obs_id_str = "%09d" % (obs_id) if isinstance(obs_id, int) else obs_id
    module_id = get_header_value("INSTRUME", header, "")
    exp_start_time = f"{get_header_value('DATE-OBS', header, '')} {get_header_value('TIME-OBS', header, '')}"
    exp_time = get_header_value("EXPTIME", header, 0)
    module_id = get_header_value("INSTRUME", header, "")
    obs_type = get_header_value("FILETYPE", header, "")
    detector = str(get_header_value("DETECTOR", header, ""))
    filename = get_header_value("FILENAME", header, "")
    mjd_int = int(get_header_value("EXPSTART", header, 0))

    if module_id.upper() == "HSTDM":
        img_header = hdul[0].header
    else:
        img_header = hdul[1].header

    filter = get_header_value("FILTER", img_header, "")
    
    #level0_headder
    object_name = get_header_value("OBJECT", header, "-")
    ra_obj = get_header_value("RA_OBJ", header, 0)
    dec_obj = get_header_value("DEC_OBJ", header, 0)
    crpix1 = get_header_value("CRPIX1", img_header, 0)
    crpix2 = get_header_value("CRPIX2", img_header, 0)
    crval1 = get_header_value("CRVAL1", img_header, 0)
    crval2 = get_header_value("CRVAL2", img_header, 0)
    ctype1 = get_header_value("CTYPE1", img_header, "")
    ctype2 = get_header_value("CTYPE2", img_header, "")
    cd1_1 = get_header_value("CD1_1", img_header, 0)
    cd1_2 = get_header_value("CD1_2", img_header, 0)
    cd2_1 = get_header_value("CD2_1", img_header, 0)
    cd2_2 = get_header_value("CD2_2", img_header, 0)
    version = get_header_value("IMG_VER", img_header, "-")
    # -----------------------------------------------------------------
    # 针对模块id特殊处理
    if len(detector) > 2 and module_id.upper() == "MSC":
        detector = detector[-2:]

    if module_id.upper() == "IFS" or module_id.upper() == "MCI" :
        obs_type = get_header_value("OBSTYPE", header, "")
        exp_start_time = get_header_value('DATE-OBS', header, '')
        ra_obj = get_header_value("OBJ_RA", header, 0)
        dec_obj = get_header_value("OBJ_DEC", header, 0)
        detector = str(get_header_value("CAMERA", img_header, ""))
        filter = get_header_value("DETNAM", img_header, "")

    if module_id.upper() == "MCI" :
        obs_type = get_header_value("OBSTYPE", header, "")
        exp_start_time = get_header_value('DATE-OBS', header, '')
        ra_obj = get_header_value("OBJ_RA", header, 0)
        dec_obj = get_header_value("OBJ_DEC", header, 0)
        detector = str(get_header_value("CAMERA", img_header, ""))

    if module_id.upper() == "CPIC":
        exp_start_time = get_header_value('DATE-OBS', header, '')
        obs_type = get_header_value("OBSTYPE", header, "")
        detector = get_header_value("CCDLABEL", img_header, "")
        if len(detector) > 2:
            detector = detector[0:2]
        object_name = get_header_value("TARGET", header, "-")

    if module_id.upper() == "HSTDM":
        if len(detector) == 1:
            detector = '0' + detector
    # -----------------------------------------------------------------
    
    qc0_status = -1
    prc_status = -1
    time_now = datetime.datetime.now()
    create_time = time_now.strftime('%Y-%m-%d %H:%M:%S')

    facility_status_id = 0
    module_status_id = 0

    existed = db.exists("select * from t_observation where obs_id=?", (obs_id_str,))
    if not existed:
        db.execute("insert into t_observation \
            (obs_id,obs_time,exp_time,module_id,obs_type,facility_status_id, module_status_id, qc0_status, prc_status,create_time) \
            values (?,?,?,?,?,?,?,?,?,?)",
        (obs_id_str,exp_start_time,exp_time,module_id,obs_type,facility_status_id,module_status_id,qc0_status, prc_status,create_time))
        db.end()
    #level0
    # 文件路径处理
    #MSC----组件名
    file_dir_in_db = f"L0/{module_id}/{obs_type.upper()}/{mjd_int}/{obs_id_str}/MS"

    if module_id.upper() == "IFS":
        file_dir_in_db = f"L0/{module_id}/{obs_type.upper()}/{mjd_int}"
    if module_id.upper() == "MCI":
        file_dir_in_db = f"L0/{module_id}/{obs_type.upper()}/{mjd_int}/{obs_id_str}"
    if module_id.upper() == "CPIC":
        file_dir_in_db = f"L0/{module_id}/{obs_type.upper()}/{mjd_int}/{detector}"
    if module_id.upper() == "HSTDM":
        file_dir_in_db = f"L0/{module_id}/{obs_type.upper()}/{mjd_int}"
    
    level0Ids = db.select_one(
            "select id from t_level0_data where filename=?",
            (filename,)
        )

    detector_status_id = 0
    
    if level0Ids is None:
        file_full_path = file_path
        if copyfiles:
            file_dir = f"{dest_root_dir}/{file_dir_in_db}"
            if not os.path.exists(file_dir):
                os.makedirs(file_dir)
            file_full_path = f"{file_dir}/{filename}"
            shutil.copyfile(file_path, file_full_path)
        level0_id = f"{obs_id}{detector}"
        c = db.execute("insert into t_level0_data \
            (level0_id, obs_id, detector_no, filter, obs_type, module_id, obs_time, exp_time,detector_status_id, filename, file_path,qc0_status, prc_status,create_time) \
            values (?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
            (level0_id, obs_id, detector, filter, obs_type, module_id, exp_start_time, exp_time, detector_status_id, filename, file_full_path, qc0_status, prc_status,create_time))
        level0_id_id = db.last_row_id()
    else:
        level0_id_id = level0Ids[0]
        log.warning('%s has already been imported' %(file_path, ))
        
    #level0-header
    db.execute("delete from t_level0_header where id=?",(level0_id_id,))
    db.execute("insert into t_level0_header \
        (id, ra_obj, dec_obj, crpix1, crpix2, crval1, crval2, \
        ctype1, ctype2, cd1_1, cd1_2, cd2_1, cd2_2, object_name, version, create_time) \
        values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
        (level0_id_id, ra_obj, dec_obj, crpix1, crpix2, crval1, crval2, ctype1, ctype2, cd1_1, cd1_2, cd2_1, cd2_2, object_name, version, create_time))     
    db.end()

    rec = Level0Record(
        id = level0_id_id,
        level0_id = level0_id,
        filter = filter,
        module_id = module_id,
        obs_id = obs_id,
        detector_no = detector,
        obs_type = obs_type,
        obs_time = exp_start_time,
        exp_time = exp_time,
        detector_status_id = detector_status_id,
        filename = filename,
        file_path = file_full_path
    )

    print(f"{file_path} imported")
    return rec

if __name__ == "__main__":
    ingest()